/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  ** This notice applies to any and all portions of this file
  * that are not between comment pairs USER CODE BEGIN and
  * USER CODE END. Other portions of this file, whether 
  * inserted by the user or by software development tools
  * are owned by their respective copyright owners.
  *
  * COPYRIGHT(c) 2018 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f4xx_hal.h"
#include "tim.h"
#include "usart.h"
#include "gpio.h"

/* USER CODE BEGIN Includes */
#include "stdlib.h"
#include "string.h"
#include "math.h"
#include "esp.h"
/* USER CODE END Includes */

/* Private variables ---------------------------------------------------------*/

/* USER CODE BEGIN PV */
/* Private variables ---------------------------------------------------------*/
char webpage[483], error[139];
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);

/* USER CODE BEGIN PFP */
/* Private function prototypes -----------------------------------------------*/

/* USER CODE END PFP */

/* USER CODE BEGIN 0 */
// Funkcja obliczajca korekcj gamma i ustawiajca jasno diody
void set_led_brightness(TIM_HandleTypeDef * timer, uint32_t channel,
		uint8_t brightness) {
	int32_t value = powf((double) brightness / 255.0, 2.2) * 49999;
	__HAL_TIM_SET_COMPARE(timer, channel, value);
}

// Funkcja ustawiajca kolor wiecenia diody RGB
void set_color(uint8_t red, uint8_t green, uint8_t blue) {
	set_led_brightness(&htim1, TIM_CHANNEL_2, red);
	set_led_brightness(&htim1, TIM_CHANNEL_3, green);
	set_led_brightness(&htim1, TIM_CHANNEL_1, blue);
}

// Funkcja wywoywana w momencie otrzymania danych przez poczenie TCP
void handle_request(UART_HandleTypeDef * uart) {
	// Odczytujemy pierwsze 6 znakw odebranego dania HTTP
	char request_begining[7];
	for (uint8_t i = 0; i < 6; i++)
		request_begining[i] = esp_recv_buffer[i];
	request_begining[6] = '\0';

	// Jeli przesane zostay parametry - pocztek dania:
	// "GET /?red=XXX&green=XXX&blue=XXX HTTP/1.1"
	if (strcmp(request_begining, "GET /?") == 0) {
		// Odczytujemy pierwsze 3 liczby, jakie pojawi si w adresie
		// - s to wartoci poszczeglnych kolorw skadowych (RGB)
		int nums[3] = { 0, 0, 0 };
		int num_counter = 0;
		uint8_t last_char_was_digit = 0;

		for (int i = 6; i < 41; i++)
			if (esp_recv_buffer[i] >= '0' && esp_recv_buffer[i] <= '9') {
				last_char_was_digit = 1;
				nums[num_counter] *= 10;
				nums[num_counter] += esp_recv_buffer[i] - '0';
			} else if (last_char_was_digit == 1) {
				last_char_was_digit = 0;
				num_counter++;
				if (num_counter == 4)
					break;
			}

		// Ustawiamy kolor na diodzie RGB
		set_color(nums[0], nums[1], nums[2]);

		// Zwracamy stron WWW z formularzem wyboru kolorw
		esp_send_data_and_close(uart, esp_recv_mux, webpage);

		// Jeli danie dotyczy strony gwnej - "GET / HTTP/1.1"
	} else if (strcmp(request_begining, "GET / ") == 0)
		esp_send_data_and_close(uart, esp_recv_mux, webpage);

	// W innym przypadku - zwracamy komunikat o bdzie (404 Not Found)
	else
		esp_send_data_and_close(uart, esp_recv_mux, error);

	// Wznawiamy odbir w przerwaniach
	esp_restart_int_recv(uart);
}
/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  *
  * @retval None
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration----------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_USART1_UART_Init();
  MX_TIM1_Init();
  /* USER CODE BEGIN 2 */
	HAL_TIMEx_PWMN_Start(&htim1, TIM_CHANNEL_1);
	HAL_TIMEx_PWMN_Start(&htim1, TIM_CHANNEL_2);
	HAL_TIMEx_PWMN_Start(&htim1, TIM_CHANNEL_3);

	strcpy(webpage, "HTTP/1.1 200 OK\r\n");
	strcat(webpage, "Content-Type: text/html\r\n");
	strcat(webpage, "Content-Lenght: 398\r\n");
	strcat(webpage, "Connection: close\r\n\r\n");
	strcat(webpage, "<!DOCTYPE html>\r\n<html>\r\n<head>\r\n");
	strcat(webpage, "<title>Dioda RGB</title>\r\n</head>\r\n");
	strcat(webpage, "<body>\r\n<form method=\"get\">\r\n");
	strcat(webpage, "<p><b>Czerwony (0-255):</b> <input type=\"text\"");
	strcat(webpage, " name=\"red\" value=\"0\" /></p>\r\n");
	strcat(webpage, "<p><b>Zielony (0-255):</b> <input type=\"text\"");
	strcat(webpage, " name=\"green\" value=\"0\" /></p>\r\n");
	strcat(webpage, "<p><b>Niebieski (0-255):</b> <input type=\"text\"");
	strcat(webpage, " name=\"blue\" value=\"0\" /></p>\r\n");
	strcat(webpage, "<input type=\"submit\" value=\"Ustaw kolor\" />\r\n");
	strcat(webpage, "</form>\r\n</body>\r\n</html>");

	strcpy(error, "HTTP/1.1 404 Not Found\r\n");
	strcat(error, "Content-Type: text/html\r\n");
	strcat(error, "Content-Lenght: 48\r\n");
	strcat(error, "Connection: close\r\n\r\n");
	strcat(error, "<html><body><h1>404 Not Found</h1></body></html>");

	//
	// DO POPRAWY!!!
	//
	if (esp_setup(&huart1, "NAZWA", "HASO"))
		set_color(0, 60, 0);
	else
		set_color(60, 0, 0);
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
	while (1) {

  /* USER CODE END WHILE */

  /* USER CODE BEGIN 3 */
		// Obsugujemy dane przychodzce przez poczenie TCP,
		// jeli w przerwaniu ustawiono flag esp_recv_flag
		if (esp_recv_flag == 1)
			handle_request(&huart1);
	}
  /* USER CODE END 3 */

}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;

    /**Configure the main internal regulator output voltage 
    */
  __HAL_RCC_PWR_CLK_ENABLE();

  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 4;
  RCC_OscInitStruct.PLL.PLLN = 100;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 4;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_3) != HAL_OK)
  {
    _Error_Handler(__FILE__, __LINE__);
  }

    /**Configure the Systick interrupt time 
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick 
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  file: The file name as string.
  * @param  line: The line in file as a number.
  * @retval None
  */
void _Error_Handler(char *file, int line)
{
  /* USER CODE BEGIN Error_Handler_Debug */
	/* User can add his own implementation to report the HAL error return state */
	while (1) {
	}
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
	/* User can add his own implementation to report the file name and line number,
	 tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
