#include "frame_parser.h"

struct frame_parser_state frame_parser_init(ws2812b_config * _led_strip_handler, uint8_t _diodes_count) {
	struct frame_parser_state state;

	state.led_strip_handler = _led_strip_handler;
	state.diodes_count = _diodes_count;
	for(uint16_t i=0; i<800; i++) state.line_buffer[i] = '\0';
	state.writer_position = 0;
	state.reader_position = 0;
	for(uint8_t i=0; i<30; i++) state.field_buffer[i] = '\0';
	state.field_position = 0;
	state.process_frame = 0;

	return state;
}

void frame_parser_recv_char(struct frame_parser_state * state, uint8_t recv_char) {
	if (state->writer_position == 0 && recv_char == '@') {
		state->writer_position++;
	} else if (state->writer_position >= 1 && state->writer_position < 799) {
		if (recv_char == '\r' || recv_char == '\n') {
			state->line_buffer[state->writer_position - 1] = '\0';
			state->writer_position = 0;
			state->process_frame = 1;
		} else {
			state->line_buffer[state->writer_position - 1] = recv_char;
			state->writer_position++;
		}
	} else {
		state->writer_position = 0;
	}
}

void frame_parser_read_field(struct frame_parser_state * state) {
	state->field_position = 0;
	while(state->line_buffer[state->reader_position] != ',' && state->line_buffer[state->reader_position] != '\0'
			&& state->field_position < 29) {
		state->field_buffer[state->field_position] = state->line_buffer[state->reader_position];
		state->reader_position++;
		state->field_position++;
	}
	state->field_buffer[state->field_position] = '\0';
	state->reader_position++;
}

void frame_parser_process_frame(struct frame_parser_state * state) {
	if(state->process_frame == 0) return;
	state->process_frame = 0;

	state->reader_position = 0;
	for(int i=0; i<30; i++) {
		ws2812b_color rgb;

		frame_parser_read_field(state);
		sscanf(state->field_buffer, "%d", &(rgb.red));

		frame_parser_read_field(state);
		sscanf(state->field_buffer, "%d", &(rgb.green));

		frame_parser_read_field(state);
		sscanf(state->field_buffer, "%d", &(rgb.blue));

		ws2812b_set_diode_color(state->led_strip_handler, i, rgb);
	}
	ws2812b_refresh(state->led_strip_handler);
}
