/**
 ******************************************************************************
 * File Name          : freertos.c
 * Description        : Code for freertos applications
 ******************************************************************************
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "FreeRTOS.h"
#include "task.h"

/* USER CODE BEGIN Includes */
#include "string.h"
#include "cmsis_os.h"
#include "usart.h"
#include "gpio.h"
/* USER CODE END Includes */

/* Variables -----------------------------------------------------------------*/

/* USER CODE BEGIN Variables */
osSemaphoreId semaphoreHandle;
osThreadId redTaskHandle;
osThreadId greenTaskHandle;
osThreadId blueTaskHandle;
/* USER CODE END Variables */

/* Function prototypes -------------------------------------------------------*/

void MX_FREERTOS_Init(void); /* (MISRA C 2004 rule 8.1) */

/* USER CODE BEGIN FunctionPrototypes */
void RedTask(void const * argument);
void GreenTask(void const * argument);
void BlueTask(void const * argument);
/* USER CODE END FunctionPrototypes */

/* Hook prototypes */

/* Init FreeRTOS */

void MX_FREERTOS_Init(void) {
	/* USER CODE BEGIN Init */
	osSemaphoreDef(semaphore);
	semaphoreHandle = osSemaphoreCreate(osSemaphore(semaphore), 1);

	osThreadDef(redTask, RedTask, osPriorityNormal, 0, 128);
	redTaskHandle = osThreadCreate(osThread(redTask), NULL);

	osThreadDef(greenTask, GreenTask, osPriorityNormal, 0, 128);
	greenTaskHandle = osThreadCreate(osThread(greenTask), NULL);

	osThreadDef(blueTask, BlueTask, osPriorityNormal, 0, 128);
	blueTaskHandle = osThreadCreate(osThread(blueTask), NULL);
	/* USER CODE END Init */
}

/* USER CODE BEGIN Application */
void RedTask(void const * argument) {
	while (1) {
		if (osSemaphoreWait(semaphoreHandle, UINT32_MAX) == osOK) {
			HAL_GPIO_WritePin(LED_RED_GPIO_Port, LED_RED_Pin, GPIO_PIN_SET);
			HAL_UART_Transmit(&huart2, "Czerwona dioda - wylaczona\r\n",
					strlen("Czerwona dioda - wylaczona\r\n"), 1000);
			osSemaphoreRelease(semaphoreHandle);
		}
		osDelay(1000);
		if (osSemaphoreWait(semaphoreHandle, UINT32_MAX) == osOK) {
			HAL_GPIO_WritePin(LED_RED_GPIO_Port, LED_RED_Pin, GPIO_PIN_RESET);
			HAL_UART_Transmit(&huart2, "Czerwona dioda - wlaczona\r\n",
					strlen("Czerwona dioda - wylaczona\r\n"), 1000);
			osSemaphoreRelease(semaphoreHandle);
		}
		osDelay(1000);
	}
}

void GreenTask(void const * argument) {
	while (1) {
		if (osSemaphoreWait(semaphoreHandle, UINT32_MAX) == osOK) {
			HAL_GPIO_WritePin(LED_GREEN_GPIO_Port, LED_GREEN_Pin, GPIO_PIN_SET);
			HAL_UART_Transmit(&huart2, "Zielona dioda - wylaczona\r\n",
					strlen("Zielona dioda - wylaczona\r\n"), 1000);
			osSemaphoreRelease(semaphoreHandle);
		}
		osDelay(2000);
		if (osSemaphoreWait(semaphoreHandle, UINT32_MAX) == osOK) {
			HAL_GPIO_WritePin(LED_GREEN_GPIO_Port, LED_GREEN_Pin,
					GPIO_PIN_RESET);
			HAL_UART_Transmit(&huart2, "Zielona dioda - wlaczona\r\n",
					strlen("Zielona dioda - wylaczona\r\n"), 1000);
			osSemaphoreRelease(semaphoreHandle);
		}
		osDelay(2000);
	}
}

void BlueTask(void const * argument) {
	while (1) {
		if (osSemaphoreWait(semaphoreHandle, UINT32_MAX) == osOK) {
			HAL_GPIO_WritePin(LED_BLUE_GPIO_Port, LED_BLUE_Pin, GPIO_PIN_SET);
			HAL_UART_Transmit(&huart2, "Niebieska dioda - wylaczona\r\n",
					strlen("Niebieska dioda - wylaczona\r\n"), 1000);
			osSemaphoreRelease(semaphoreHandle);
		}
		osDelay(3000);
		if (osSemaphoreWait(semaphoreHandle, UINT32_MAX) == osOK) {
			HAL_GPIO_WritePin(LED_BLUE_GPIO_Port, LED_BLUE_Pin, GPIO_PIN_RESET);
			HAL_UART_Transmit(&huart2, "Niebieska dioda - wlaczona\r\n",
					strlen("Niebieska dioda - wylaczona\r\n"), 1000);
			osSemaphoreRelease(semaphoreHandle);
		}
		osDelay(3000);
	}
}
/* USER CODE END Application */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
