/**
 ******************************************************************************
 * File Name          : freertos.c
 * Description        : Code for freertos applications
 ******************************************************************************
 ******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "FreeRTOS.h"
#include "task.h"

/* USER CODE BEGIN Includes */
#include "string.h"
#include "cmsis_os.h"
#include "usart.h"
#include "gpio.h"
/* USER CODE END Includes */

/* Variables -----------------------------------------------------------------*/

/* USER CODE BEGIN Variables */
osMessageQId queueHandle;
osThreadId senderTaskHandle;
osThreadId receiverTaskHandle;
/* USER CODE END Variables */

/* Function prototypes -------------------------------------------------------*/

void MX_FREERTOS_Init(void); /* (MISRA C 2004 rule 8.1) */

/* USER CODE BEGIN FunctionPrototypes */
void SenderTask(void const * argument);
void ReceiverTask(void const * argument);
/* USER CODE END FunctionPrototypes */

/* Hook prototypes */

/* Init FreeRTOS */

void MX_FREERTOS_Init(void) {
	/* USER CODE BEGIN Init */
	osMessageQDef(queue, 32, uint8_t);
	queueHandle = osMessageCreate(osMessageQ(queue), NULL);

	osThreadDef(senderTask, SenderTask, osPriorityNormal, 0, 128);
	senderTaskHandle = osThreadCreate(osThread(senderTask), NULL);

	osThreadDef(receiverTask, ReceiverTask, osPriorityNormal, 0, 128);
	receiverTaskHandle = osThreadCreate(osThread(receiverTask), NULL);
	/* USER CODE END Init */
}

/* USER CODE BEGIN Application */
void SenderTask(void const * argument)
{
	uint32_t i = 0;
	while(1)
	{
		osMessagePut(queueHandle, i, 1000);
		osDelay(1000);
		i++;
	}
}

void ReceiverTask(void const * argument)
{
	osEvent ev;
	char tmp[6];
	while(1) {
		ev = osMessageGet(queueHandle, 1000);
		if(ev.status = osEventMessage)
		{
			sprintf(tmp, "%d\r\n", ev.value.v);
			HAL_UART_Transmit(&huart2, tmp, strlen(tmp), 1000);
		}
	}
}
/* USER CODE END Application */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
