#include "lighting.h"

lighting_config lighting_init(uint16_t diodes_count) {
	lighting_config config;
	config.diodes_count = diodes_count;
	config.colors_array = calloc(diodes_count, sizeof(lighting_rgb));
	return config;
}

lighting_rgb lighting_hsv2rgb(lighting_hsv in) {
	if (in.hue >= 360) in.hue = 0;

	double c = in.value * in.saturation;
	double x = c * (1 - fabs(fmod((in.hue / 60), 2) - 1));
	double m = in.value - c;

	lighting_rgb out;
	switch ((int) (in.hue / 60)) {
		case 0: out.red = c + m; out.green = x + m; out.blue = 0; break;
		case 1: out.red = x + m; out.green = c + m; out.blue = 0; break;
		case 2: out.red = 0; out.green = c + m; out.blue = x + m; break;
		case 3: out.red = 0; out.green = x + m; out.blue = c + m; break;
		case 4: out.red = x + m; out.green = 0; out.blue = c + m; break;
		case 5: out.red = c + m; out.green = 0; out.blue = x + m; break;
	}
	return out;
}

void lighting_set_diode_color_rgb(lighting_config * config, int diode_no, lighting_rgb rgb) {
	config->colors_array[diode_no].red = rgb.red;
	config->colors_array[diode_no].green = rgb.green;
	config->colors_array[diode_no].blue = rgb.blue;
}

void lighting_set_diode_color_hsv(lighting_config * config, int diode_no, lighting_hsv hsv) {
	lighting_rgb rgb = lighting_hsv2rgb(hsv);
	lighting_set_diode_color_rgb(config, diode_no, rgb);
}

void lighting_draw_gradient_rgb(lighting_config * config, int from_diode,
		int to_diode, lighting_rgb from_color, lighting_rgb to_color) {
	for (int i = from_diode; i <= to_diode; i++) {
		double percent = (double) (i - from_diode) / (double) (to_diode - from_diode);
		lighting_rgb color;
		color.red = from_color.red + percent * (to_color.red - from_color.red);
		color.green = from_color.green + percent * (to_color.green - from_color.green);
		color.blue = from_color.blue + percent * (to_color.blue - from_color.blue);
		lighting_set_diode_color_rgb(config, i, color);
	}
}

void lighting_draw_gradient_hsv(lighting_config * config, int from_diode,
		int to_diode, lighting_hsv from_color, lighting_hsv to_color) {
	lighting_rgb from_rgb = lighting_hsv2rgb(from_color);
	lighting_rgb to_rgb = lighting_hsv2rgb(to_color);
	lighting_draw_gradient_rgb(config, from_diode, to_diode, from_rgb, to_rgb);
}

double lighting_gamma_correction(double in) {
	return powf(in, 2.2);
}

void lighting_update_ws2812b(lighting_config * lighting_c,
		ws2812b_config * ws2812b_c) {
	for (int i = 0; i < ws2812b_c->diodes_count; i++) {
		ws2812b_c->colors_array[i].red = (uint8_t) (lighting_gamma_correction(lighting_c->colors_array[i].red) * 255);
		ws2812b_c->colors_array[i].green = (uint8_t) (lighting_gamma_correction(lighting_c->colors_array[i].green) * 255);
		ws2812b_c->colors_array[i].blue = (uint8_t) (lighting_gamma_correction(lighting_c->colors_array[i].blue) * 255);
	}
}
